<?php

namespace Drupal\fullcalendar\Controller;

use Drupal\Component\Utility\Html;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Link;
use Drupal\Core\Render\Renderer;
use Drupal\Core\Url;
use Drupal\datetime\Plugin\Field\FieldType\DateTimeItemInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller to drop an entity from the calendar.
 */
class UpdateController extends ControllerBase {

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected Renderer $renderer;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    $instance = parent::create($container);
    $instance->renderer = $container->get('renderer');
    return $instance;
  }

  /**
   * Drops an entity from the calendar.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity.
   * @param string $field
   *   The field name.
   * @param int $index
   *   The index.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The json response.
   */
  public function drop(EntityInterface $entity, string $field, int $index, Request $request): JsonResponse {
    if ($request->request->has('day_delta') && $request->request->has('minute_delta')) {
      $day_delta = Html::escape($request->request->get('day_delta'));
      $minute_delta = Html::escape($request->request->get('minute_delta'));
      $delta = " $day_delta days $minute_delta minutes";

      $field_item = $entity->{$field}->get($index);
      $value = $field_item->value;
      $field_item->set('value', date(DateTimeItemInterface::DATETIME_STORAGE_FORMAT, strtotime($value . $delta)));

      // Save the new start/end values.
      $entity->save();

      $url = Url::fromUserInput('/');
      $link = Link::fromTextAndUrl($this->t('Close'), $url);
      $link = $link->toRenderable();
      $link['#attributes']['class'][] = 'fullcalendar-status-close';

      $message = $this->t('The new event time has been saved.');
      $message .= ' [' . $this->renderer->render($link) . ']';
    }
    else {
      $message = $this->t('The event has not been updated.');
    }

    return new JsonResponse([
      'msg'    => $message,
      'dom_id' => $request->request->get('dom_id'),
    ]);
  }

}
